/*********************************************************************
*                SEGGER Microcontroller GmbH & Co. KG                *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 2003 - 2021  SEGGER Microcontroller GmbH & Co. KG        *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
----------------------------------------------------------------------
File        : Main.h
Purpose     : Main header
---------------------------END-OF-HEADER------------------------------
*/

#ifndef MAIN_H
#define MAIN_H

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "DIALOG.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
//
// Demo IDs
//
#define ID_GAME_ARCHERY           0
#define ID_GAME_RUNNING           1
#define ID_GAME_WEIGHTLIFTING     2
#define ID_GAME_CYCLING           3
//
// General
//
#define DEMO_GAME_COUNT           4
#define STATWIN_BORDER            20
#define ANIM_PERIOD_STATS         300
#define DEMO_TITLE                "SPORTS GAMES"
//
// Util
//
#define RAND(Min, Max)              APP_GetRandomNumber(Min, Max)
#define MAGNIFY_RECT(r, f)          GUI_AddRect(r, r, f)
#define SHRINK_RECT(r, f)           GUI_AddRect(r, r, -f)
#define TIME_TO_MS(i)               ((i % 1000) / 10)
#define TIME_TO_SEC(i)              ((i / 1000) % 60)
#define TIME_TO_MIN(i)              ((i / 1000) / 60)
#define MS_TO_AVG_KMH(m, t)         ((t < 1000) ? 0 : ((((m * 100000) / t) * 36) / 1000))
#define PRINT_TIME(ps, t)           sprintf(ps, "%02d:%02d:%02d", TIME_TO_MIN(t), TIME_TO_SEC(t), TIME_TO_MS(t))
#define PRINT_POINTS(ps, x)         sprintf(ps, "%d", x)
#define LCD_X                       480
#define LCD_Y                       272
#define PERCENTAGE(PctVal, BaseVal) (((PctVal * 10000) / BaseVal) / 100)
//
// Brandings
//
#define RENESAS_BRANDING            0
#define SEGGER_BRANDING             1

/*********************************************************************
*
*       Defines, configurable
*
**********************************************************************
*/
//
// Branding
//
#define BRANDING                SEGGER_BRANDING
//
// Highscore related
//
#define MAX_NAME_LENGTH         10
#define MAX_HIGHSCORE_ENTRIES   10

/*********************************************************************
*
*       Types
*
**********************************************************************
*/
//
// Main Menu
//
typedef struct {
  WM_HWIN hWin;
  int     xSize;
  int     xPos;
  int     yPos;
} ANIM_TITLE_DATA;

typedef struct {
  WM_HWIN hWin;
  int     xDist;
  int     xStart;
  int     xPos;
  int     Dir;
} ANIM_DATA_MENU;

//
// Prototype for creating game window
//
typedef WM_HWIN (* DEMO_FUNC)(char * acName);

//
// Highscore types
//
typedef struct {
  int  Data;
  char acName[MAX_NAME_LENGTH + 1];
} HIGHSCORE_ENTRY;

typedef struct {
  HIGHSCORE_ENTRY aEntries[MAX_HIGHSCORE_ENTRIES];
  unsigned        Count;
} HIGHSCORE_LIST;

//
// Running & Cycling
//
typedef struct {
  int Meters;
  int TimeStart;
  int FinalTime;
  int KmhAvg;
  int KmhBest;
} STATS;

typedef struct {
  WM_HWIN hWin;
  int     xPos;
  int     yPos;
  int     xDist;
  int     yDist;
  int     xSizeOrg;
  int     ySizeOrg;
  int     xSizeNew;
  int     ySizeNew;
} STAT_WIN_ANIM_DATA;

/*********************************************************************
*
*       Prototypes
*
**********************************************************************
*/
//
// Create demo games
//
WM_HWIN ARCHERY_Create         (char * acName);
WM_HWIN RUNNING_Create         (char * acName);
WM_HWIN WEIGHTLIFTING_Create   (char * acName);
WM_HWIN CYCLING_Create         (char * acName);
//
// General functions
//
void    APP_AddHighscoreEntry  (const HIGHSCORE_ENTRY * pEntry, unsigned GameIndex);
void    APP_AddHighscoreScore  (int Data, unsigned GameIndex);
WM_HWIN APP_CreateMainMenu     (void);
WM_HWIN APP_CreateMenuButton   (WM_HWIN hParent);
int     APP_GetRandomNumber    (int min, int max);
void    APP_RemoveCurrentWindow(void);

#endif /* MAIN_H */

/*************************** End of file ****************************/